package com.example.leeyuanair.englishhelper;

import android.app.Activity;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.content.res.AssetManager;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;
import android.widget.RadioButton;
import android.widget.TextView;
import android.widget.Toast;

import com.example.leeyuanair.englishhelper.access.WordInfo;
import com.example.leeyuanair.englishhelper.act_allwords.ActWordInfo;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collections;

public class ActSearchWord extends AppCompatActivity implements RadioButton.OnClickListener, AdapterView.OnItemClickListener {
    //Valuables
    WordInfo[] words;              //所有的單字與其資訊
    String selects;                 //表示使用者所選的篩選選項。"all"=全部, "n"=名詞, "adj"=形容詞, "v"=動詞, "adv"=副詞, "prep"=介係詞
    ArrayList<String> finds;       //所選清單，表示搜尋時所找到的單字清單。
    ArrayAdapter<String> wordlist; //連結ArrayList與清單物件lstFinds之間的物件
    boolean isAddingMode;       //判斷是否為Group增加單字的模式
    ArrayList<String> Addition;     //紀錄使用者所要增加的單字
    //RadioButton
    RadioButton rdoAll, rdoNoun, rdoAdjective, rdoVerb, rdoAdverb, rdoPreposition;
    //Button
    Button btnSearch;
    //TextView
    TextView txtSearch;
    //ListView
    ListView lstFinds;
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_act_search_word);
        setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
        setTitle("單字查詢");
        //初始化變數
        selects = "all";
        finds = new ArrayList<>();
        wordlist = new ArrayAdapter<String>(this,android.R.layout.simple_list_item_1,finds);
        //鏈結btnSearch元件
        btnSearch = (Button) findViewById(R.id.btnSearch);
        //鏈結txtSearch元件
        txtSearch = (TextView) findViewById(R.id.txtSearch);
        //鏈結lstFinds元件，並且與wordlist連結、登錄事件
        lstFinds = (ListView) findViewById(R.id.lstFinds);
        lstFinds.setAdapter(wordlist);
        lstFinds.setOnItemClickListener(this);
        //鏈結所有的RadioButton元件
        rdoAll = (RadioButton) findViewById(R.id.rdoAll);
        rdoNoun = (RadioButton) findViewById(R.id.rdoNoun);
        rdoAdjective = (RadioButton) findViewById(R.id.rdoAdjective);
        rdoVerb = (RadioButton) findViewById(R.id.rdoVerb);
        rdoAdverb = (RadioButton) findViewById(R.id.rdoAdverb);
        rdoPreposition = (RadioButton) findViewById(R.id.rdoPreposition);
        //註冊RadioButton事件
        rdoAll.setOnClickListener(this);
        rdoNoun.setOnClickListener(this);
        rdoAdjective.setOnClickListener(this);
        rdoVerb.setOnClickListener(this);
        rdoAdverb.setOnClickListener(this);
        rdoPreposition.setOnClickListener(this);
        //讀取所有單字
        AssetManager am = getAssets();
        try {
            InputStream is = am.open("Words.txt");
            byte[] data = new byte[is.available()];
            is.read(data); is.close();
            words = WordInfo.analyzeWordInfoString(new String(data));
        }catch(Exception ex){
            Toast.makeText(this,"資料讀取錯誤",Toast.LENGTH_SHORT).show();
        }
        //判別是否為增加單字模式
        isAddingMode = (boolean) getIntent().getExtras().get("isAddingMode");
        Addition = new ArrayList<>();
    }

    //返回上一層頁面
    public void btnBack2_Click(View view) {
        Intent theIntent = new Intent();
        theIntent.putExtra("Addition", Addition);
        setResult(0, theIntent);
        finish();
    }

    //當【搜尋】按鈕按下時 - 執行查詢單字的功能
    public void btnSearch_Click(View view) {
        InputMethodManager inputMethodManager = (InputMethodManager)  this.getSystemService(Activity.INPUT_METHOD_SERVICE); //取得輸入介面物件
        inputMethodManager.hideSoftInputFromWindow(this.getCurrentFocus().getWindowToken(), 0);                                    //隱藏螢幕鍵盤
        String target = txtSearch.getText().toString();          //取得使用者所輸入的文字
        boolean isEnglish = checkEnglish(target.toCharArray()); //判斷使用者輸入的文字是否為英文
        //讀取所有單字資料
        try{
            String[] spis;
            finds.clear();
            //依條件尋找單字
            if(isEnglish){  //若使用者輸入的字串為英文時
                //循序尋找有沒有(部分)符合的字串與詞性
                for(WordInfo theWord : words){
                    if(theWord.word.contains(target)){     //若使用者所輸入的字與所選單字有部分符合時
                        if(checkPartofSpeech(theWord)){    //若該單字有使用者所指定的詞性時
                            finds.add(theWord.word);
                        }
                    }
                }
            }else{          //若使用者輸入的字串為非英文時
                //循序尋找是否與使用者所選的詞性相符，且與使用者所輸入的文字有相關的單字
                for(WordInfo theWord : words){
                    if(checkPartofMean(theWord, target)){
                        finds.add(theWord.word);
                    }
                }
            }
            //將所找到的資料更新至lstFinds
            if(finds.isEmpty())Toast.makeText(this,"未找到任何訊息相符的單字。",Toast.LENGTH_SHORT).show();
            wordlist.notifyDataSetChanged();
        }catch(Exception ex){
            Toast.makeText(this,ex.toString(),Toast.LENGTH_SHORT).show();
        }
    }

    //檢測是否為純英文單字字串
    boolean checkEnglish(char[] input){
        boolean isEnglish = true;
        for(int i=0;i<input.length;i++) {
            if ((input[i]<65 || input[i]>90) && (input[i]<97 || input[i]>122)) {
                isEnglish = false;
                break;
            }
        }
        return isEnglish;
    }

    //判斷該單字是否有某指定的詞性
    boolean checkPartofSpeech(WordInfo theWord){
        boolean returns = false;
        if(selects.equals("all")){
            returns = true;
        }else{
            switch (selects){
                case "n" :
                    returns = theWord.noun!=null;
                    break;
                case "a" :
                    returns = theWord.adjective!=null;
                    break;
                case "v" :
                    returns = theWord.verb!=null;
                    break;
                case "adv" :
                    returns = theWord.adverb!=null;
                    break;
                case "prep" :
                    returns = theWord.preposition!=null;
                    break;
                default :
            }
        }
        return returns;
    }

    //判斷是否與使用者所選的詞性相符，且與使用者所輸入的文字有相關的單字
    boolean checkPartofMean(WordInfo theWord,String target){
        boolean returns = false;
        if(selects.equals("all")){
            returns = (theWord.noun!=null) && (theWord.noun.contains(target)) ||
                      (theWord.adjective!=null) && (theWord.adjective.contains(target)) ||
                      (theWord.verb!=null) && (theWord.verb.contains(target)) ||
                      (theWord.adverb!=null) && (theWord.adverb.contains(target)) ||
                      (theWord.preposition!=null) && (theWord.preposition.contains(target));
        }else{
            switch (selects){
                case "n" :
                    returns = (theWord.noun!=null) && (theWord.noun.contains(target));
                    break;
                case "a" :
                    returns = (theWord.adjective!=null) && (theWord.adjective.contains(target));
                    break;
                case "v" :
                    returns = (theWord.verb!=null) && (theWord.verb.contains(target));
                    break;
                case "adv" :
                    returns = (theWord.adverb!=null) && (theWord.adverb.contains(target));
                    break;
                case "prep" :
                    returns = (theWord.preposition!=null) && (theWord.preposition.contains(target));
                    break;
                default :
            }
        }
        return returns;
    }

    //RadioButton事件 - 代表唯一選取的功能
    @Override
    public void onClick(View v) {
        //判斷呼叫事件的物件類別是否為AppCompatRadioButton，也就是單選按鈕。
        if(v.getClass().getSimpleName().equals("AppCompatRadioButton")){
            uncheckAllRadioButton();
            ((RadioButton) v).setChecked(true);
            if(v==rdoAll)selects="all";
            else if(v==rdoNoun)selects="n";
            else if(v==rdoAdjective)selects="a";
            else if(v==rdoVerb)selects="v";
            else if(v==rdoAdverb)selects="adv";
            else if(v==rdoPreposition)selects="prep";
        }
    }

    //程序 - 解除所有RadioButton的選取
    void uncheckAllRadioButton(){
        rdoAll.setChecked(false);
        rdoNoun.setChecked(false);
        rdoAdjective.setChecked(false);
        rdoVerb.setChecked(false);
        rdoAdverb.setChecked(false);
        rdoPreposition.setChecked(false);
    }

    //當使用者按下清單裡的某一項時，即顯示使用者選所的單字訊息。
    //若模式為新增群組單字模式時，則傳遞新增單字的布林值true至下個頁面
    @Override
    public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
        try{
            Object SelItem = parent.getItemAtPosition(position);
            Intent theIntent = new Intent(this, ActWordInfo.class);
            theIntent.putExtra("SelectedWord", SelItem.toString());
            theIntent.putExtra("isAddingMode", isAddingMode);
            startActivityForResult(theIntent, 0);
        }catch(Exception ex){
            Toast.makeText(this,ex.toString(),Toast.LENGTH_SHORT).show();
        }
    }
    //當下一層的頁面關閉時
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if(resultCode==0){
            String theWord = data.getExtras().getString("isAdd");
            //若使用者有選擇該單字且該單字之前沒有加入過群組的話則將單字加入
            if(theWord!=null && !Addition.contains(theWord)){
                Addition.add(theWord);
                Collections.sort(Addition);
            }
        }
    }

    //攔截使用者所按下的按鍵 - 包含返回、家、視窗等等按鍵
    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event){
        if(keyCode == KeyEvent.KEYCODE_BACK){
            Intent theIntent = new Intent();
            theIntent.putExtra("Addition", Addition);
            setResult(0, theIntent);
            finish();
        }
        return super.onKeyDown(keyCode, event);
    }
}
